﻿using System.Collections.Generic;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Media;

using DevExpress.Xpf.Editors;

namespace TestLibrary
{
    /// <summary>
    /// 카메라 컨트롤
    /// </summary>
    public partial class CameraControl : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 캡처 여부
        /// </summary>
        private bool isCapturing = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 장치명 - DeviceName

        /// <summary>
        /// 장치명
        /// </summary>
        public string DeviceName
        {
            get
            {
                return this.cameraControl.DeviceInfo.Name; 
            }
            set
            {
                DeviceInfo deviceInfo = GetDeviceInfo(value);

                this.cameraControl.DeviceInfo = deviceInfo;
            }
        }

        #endregion
        #region 캡처 여부 - IsCapturing

        /// <summary>
        /// 캡처 여부
        /// </summary>
        public bool IsCapturing
        {
            get
            {
                return this.isCapturing;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CameraControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public CameraControl()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 장치명 리스트 구하기 - GetDeviceNameList()

        /// <summary>
        /// 장치명 리스트 구하기
        /// </summary>
        /// <returns></returns>
        public static List<string> GetDeviceNameList()
        {
            List<DeviceInfo> deviceInfoList = GetDeviceInfoList();

            List<string> list = new List<string>();

            foreach(DeviceInfo deviceInfo in deviceInfoList)
            {
                list.Add(deviceInfo.Name);
            }

            return list;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// private

        #region 장치 정보 리스트 구하기 - GetDeviceInfoList()

        /// <summary>
        /// 장치 정보 리스트 구하기
        /// </summary>
        /// <returns>장치 정보 리스트</returns>
        private static List<DeviceInfo> GetDeviceInfoList()
        {
            DevExpress.Xpf.Editors.CameraControl control = new DevExpress.Xpf.Editors.CameraControl();

            List<DeviceInfo> deviceInfoList = control.GetAvailableDevices().ToList();

            return deviceInfoList;
        }

        #endregion
        #region 장치 정보 구하기 - GetDeviceInfo(deviceName)

        /// <summary>
        /// 장치 정보 구하기
        /// </summary>
        /// <param name="deviceName">장치명</param>
        /// <returns>장치 정보</returns>
        private static DeviceInfo GetDeviceInfo(string deviceName)
        {
            if(string.IsNullOrEmpty(deviceName))
            {
                return null;
            }

            List<DeviceInfo> deviceInfoList = GetDeviceInfoList();

            DeviceInfo deviceInfo = deviceInfoList.Where(d => d.Name == deviceName).FirstOrDefault();

            return deviceInfo;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 시작하기 - Start()

        /// <summary>
        /// 시작하기
        /// </summary>
        public void Start()
        {
            if(this.isCapturing)
            {
                return;
            }

            this.cameraControl.Start();

            this.isCapturing = true;
        }

        #endregion
        #region 중단하기 - Stop()

        /// <summary>
        /// 중단하기
        /// </summary>
        public void Stop()
        {
            if(!this.isCapturing)
            {
                return;
            }

            this.cameraControl.Stop();

            this.isCapturing = false;
        }

        #endregion
        #region 이미지 구하기 - GetImage()

        /// <summary>
        /// 이미지 구하기
        /// </summary>
        /// <returns>이미지 소스</returns>
        public ImageSource GetImage()
        {
            return this.cameraControl.TakeSnapshot();
        }

        #endregion
    }
}